@@ -4,9 +4,8 @@ namespace EshopCatalog\AdminModule\Model\Import;
 
 use Core\Model\Helpers\BaseService;
 use Core\Model\Lang\DefaultLang;
-use Doctrine\DBAL\Exception\UniqueConstraintViolationException;
 use Doctrine\ORM\Query;
-use EshopCatalog\Model\Entities\ManufacturerTexts;
+use EshopCatalog\Model\Entities\CategoryExport;
 use EshopCatalog\Model\Entities\Variant;
 use EshopCatalog\Model\Entities\VariantValue;
 use EshopCatalog\Model\Entities\VariantValueText;
@@ -25,6 +24,7 @@ use EshopCatalog\Model\Entities\ProductVariantCombination;
 use EshopCatalog\Model\Entities\ProductVariantSupplier;
 use EshopCatalog\Model\Entities\Supplier;
 use EshopCatalog\Model\Entities\VatRate;
+use EshopCatalog\Model\Helpers\ExportEnums;
 use Gallery\Model\Entities\Album;
 use Gallery\Model\Entities\Image;
 use Import\AdminModule\Model\DictionaryResolver;
@@ -51,6 +51,9 @@ class ImportHelper extends BaseService
 	/** @var Manufacturer[] */
 	protected $cManufacturers;
 
+	/** @var CategoryProduct[] */
+	protected $cCategoryProduct;
+
 	/** @var Supplier[] */
 	protected $cSuppliers;
 
@@ -201,6 +204,7 @@ class ImportHelper extends BaseService
 		$parent = $baseCat;
 		$cat    = null;
 
+		$this->cCats = [];
 		for ($i = 0; $i < count($deep); $i++) {
 			$cat      = null;
 			$name     = trim($deep[$i]);
@@ -217,17 +221,25 @@ class ImportHelper extends BaseService
 			}
 
 			if ($found) {
-				$cat = $this->em->getReference(Category::class, $found['id']);
+				$cat = $this->em->getRepository(Category::class)->find($found['id']);
 
 				$this->cCats[$cCatsKey] = $cat;
 			} else {
 				$cat = new Category();
 				$cat->setParent($parent);
+				$this->em->persist($cat)->flush($cat);
+
 				$ct = new CategoryTexts($cat, $lang);
 				$ct->setName($name);
-				$this->em->persist($ct);
-				$this->em->persist($cat)->flush();
+				$this->em->persist($ct)->flush($ct);
 				$this->cCats[$cCatsKey] = $cat;
+
+				foreach (ExportEnums::$services as $k => $v) {
+					$cp         = new CategoryExport($cat, $lang, $k);
+					$cp->status = 0;
+					$this->em->persist($cp);
+					$this->em->flush($cp);
+				}
 			}
 			$parent = $cat;
 		}
@@ -235,6 +247,24 @@ class ImportHelper extends BaseService
 		return $cat;
 	}
 
+	public function checkCategoryProductExist($productId, $categoryId)
+	{
+		$key = "$productId-$categoryId";
+		$cat = isset($this->cCategoryProduct[$key]) ? $this->cCategoryProduct[$key] : null;
+
+		if (!$cat) {
+			$cat = $this->em->getRepository(CategoryProduct::class)->findOneBy([
+				'product'  => $productId,
+				'category' => $categoryId,
+			]);
+
+			if ($cat)
+				$this->cCategoryProduct[$key] = $cat;
+		}
+
+		return $cat ? true : false;
+	}
+
 	/**
 	 * @param Product  $product
 	 * @param Category $category
@@ -243,13 +273,22 @@ class ImportHelper extends BaseService
 	 */
 	public function getCategoryProduct($product, $category)
 	{
-		$cat = $this->em->getRepository(CategoryProduct::class)->findOneBy([
-			'product'  => $product,
-			'category' => $category,
-		]);
-		if (!$cat)
-			$cat = new CategoryProduct($product, $category);
-		$this->em->persist($cat);
+		$key = $product->getId() . '-' . $category->getId();
+		$cat = isset($this->cCategoryProduct[$key]) ? $this->cCategoryProduct[$key] : null;
+
+		if (!$cat) {
+			$cat = $this->em->getRepository(CategoryProduct::class)->findOneBy([
+				'product'  => $product,
+				'category' => $category,
+			]);
+
+			if (!$cat) {
+				$cat = new CategoryProduct($product, $category);
+				$this->em->persist($cat);
+			}
+
+			$this->cCategoryProduct[$key] = $cat;
+		}
 
 		return $cat;
 	}
@@ -258,12 +297,14 @@ class ImportHelper extends BaseService
 	 * @param Product $product
 	 *
 	 * @return Album
+	 * @throws \Exception
 	 */
 	public function getGallery($product)
 	{
 		if (!$product->gallery) {
 			$gallery = new Album(UPLOADS_PATH . '/products');
 			$this->em->persist($gallery);
+			$this->em->flush($gallery);
 
 			return $gallery;
 		}
@@ -271,6 +312,27 @@ class ImportHelper extends BaseService
 		return $product->gallery;
 	}
 
+	public function getImages($data, ?string $urlPrefix = ''): array
+	{
+		$imgs = [];
+		$loop = function($rows) use (&$loop, &$imgs, $urlPrefix) {
+			if (is_string($rows))
+				$imgs[] = $urlPrefix . $rows;
+			else
+				foreach ($rows as $v) {
+					if (is_array($v))
+						$loop($v);
+
+					if (is_string($v))
+						$imgs[] = $urlPrefix . $v;
+				}
+		};
+
+		$loop($data);
+
+		return $imgs;
+	}
+
 	/**
 	 * @param Album  $gallery
 	 * @param string $url
@@ -302,13 +364,28 @@ class ImportHelper extends BaseService
 
 			if (!$exist) {
 				// TODO dát do služby a nastavit i v imagesZone komponentě
-				$image = new Image($gallery, $basename);
-				$this->em->persist($image);
+				$lastPosition = $this->em->getConnection()
+					->fetchColumn("SELECT position FROM gallery__image WHERE album_id = :album", [':album' => $gallery->getId()]);
+				if (!$lastPosition && $lastPosition != 0)
+					$position = 0;
+				else
+					$position = $lastPosition + 1;
+				$this->em->getConnection()->insert('gallery__image', [
+					'filename'     => $basename,
+					'path'         => $gallery->generatePath(),
+					'album_id'     => $gallery->getId(),
+					'position'     => $position,
+					'is_published' => 1,
+					'is_cover'     => 0,
+				]);
+				$imageId = $this->em->getConnection()->lastInsertId();
 
-				$filename        = $image->getId() . '_' . $basename;
-				$image->filename = $filename;
-				$this->em->persist($image);
-				$gallery->addImage($image);
+				$filename = $imageId . '_' . $basename;
+				$this->em->getConnection()->update('gallery__image', [
+					'filename' => $filename,
+				], ['id' => $imageId]);
+
+//				$gallery->addImage($this->em->getReference(Image::class, $imageId));
 
 				$dir = WWW_DIR . $gallery->generatePath();
 				FileSystem::createDir($dir);
@@ -317,11 +394,21 @@ class ImportHelper extends BaseService
 				$this->onError[] = function() use ($dir, $filename) {
 					FileSystem::delete($dir . '/' . $filename);
 
-					if (count(scandir($dir)) == 2)
+					if (count(scandir($dir)) == 2) {
 						FileSystem::delete($dir);
+
+						$galleryId = (int) basename($dir);
+						$gallery   = $this->em->getRepository(Album::class)->find($galleryId);
+						if ($gallery) {
+							$this->em->remove($gallery);
+						}
+					}
 				};
 			}
 		} catch (\Exception $e) {
+			echo "<pre>";
+			var_dump($e);
+			die();
 		}
 	}
 
@@ -330,10 +417,18 @@ class ImportHelper extends BaseService
 	 */
 	public function moveGalleryImage($galleries)
 	{
+		return;
 		foreach ($galleries as $gallery) {
+			if (!$gallery || $gallery instanceof Album === false)
+				continue;
+
 			$gallery = $this->em->getRepository(Album::class)->find($gallery->getId());
-			$path    = $gallery->generatePath();
-			$dir     = WWW_DIR . $path;
+
+			if (!$gallery)
+				continue;
+
+			$path = $gallery->generatePath();
+			$dir  = WWW_DIR . $path;
 			FileSystem::createDir($dir);
 
 			foreach ($gallery->getImages() as $img) {
@@ -546,11 +641,13 @@ class ImportHelper extends BaseService
 		$feature = new Feature();
 		$feature->setIsPublished(1);
 		$feature->setUseAsFilter(0);
+		$this->em->persist($feature);
+
 		$featureText       = new FeatureTexts($feature, $this->lang);
 		$featureText->name = $value;
+		$this->em->persist($featureText);
 
-		$this->em->persist($feature)
-			->persist($featureText);
+		$this->em->flush([$feature, $featureText]);
 
 		return $feature;
 	}
@@ -567,11 +664,13 @@ class ImportHelper extends BaseService
 	{
 		$featureValue = new FeatureValue();
 		$featureValue->setFeature(is_numeric($featureId) ? $this->em->getReference(Feature::class, $featureId) : $featureId);
+		$this->em->persist($featureValue);
+
 		$featureValueText       = new FeatureValueTexts($featureValue, $this->lang);
 		$featureValueText->name = $value;
+		$this->em->persist($featureValueText);
 
-		$this->em->persist($featureValue)
-			->persist($featureValueText);
+		$this->em->flush([$featureValue, $featureValueText]);
 
 		return $featureValue;
 	}
